#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <limits.h>
#include <math.h>
#include <sys/time.h>


#if (defined(__unix__) || defined(unix)) && !defined(USG)
#include <sys/param.h> /* include BSD's version infomation */
#endif

#if (defined(BSD) && (BSD >= 199306)) /* for 4.4 BSD code-base or newer */
#include <machine/soundcard.h>
#else
#include <linux/soundcard.h>
#endif

#include "xvsnd.h"

#ifndef PI
#define PI 3.141592653589793
#endif

int dsp_wopen(char device[])
/* dspǥХ񤭹Ѥǥץ */
{
  int wdsp;

  if((wdsp=open(device, O_WRONLY, 0)) == -1){
    fprintf(stderr, "Can't open \"%s\"\n", device);
    return -1;
  }
  return wdsp;
}
  
int dsp_ropen(char device[])
/* dspǥХɤ߹Ѥǥץ */
{
  int rdsp;

  if((rdsp=open(device, O_RDONLY, 0)) == -1){
    fprintf(stderr, "Can't open \"%s\"\n", device);
    return -1;
  }
  return rdsp;
}

snd_head set_dsp_param(int fp, snd_head wav)
/* dspΥѥ᡼ѹ ̻Ҳӥåȿ -> ͥ -> ץ󥰼ȿ */
{
  int srbset, chset, sfset;
  snd_head set_wav;

  srbset = wav.srb;
  sfset = wav.sf;
  chset = wav.ch;

  if(srbset != 0){
    if(ioctl(fp, SOUND_PCM_WRITE_BITS, &srbset) == -1){
      fprintf(stderr, "Error. Can't change sampling resolution to %d bits\n",
	      wav.srb);
      exit(-1);
    }
    if(wav.srb != srbset){
      fprintf(stderr, "Warning!! Sampling resolution is %d bits instead of %d bits\n", srbset, wav.srb);
    }
    set_wav.srb = srbset;
  }

  if(chset != 0){
    if(ioctl(fp, SOUND_PCM_WRITE_CHANNELS, &chset) == -1){
      fprintf(stderr, "Error. Can't change %d channel(s)\n", wav.ch);
      exit(-1);
    }
    if(wav.ch != chset){
      fprintf(stderr, "Warning!! %d channel(s) instead of %d channel(s)\n", chset, wav.ch);
    }
    set_wav.ch = chset;
  }

  if(sfset != 0){
    if(ioctl(fp, SOUND_PCM_WRITE_RATE, &sfset) == -1){
      fprintf(stderr, "Error. Can't change sampling frequency to %d Hz\n", wav.sf);
      exit(-1);
    }
    if(wav.sf != sfset){
      fprintf(stderr, "Warning!! Sampling freq. is %d Hz instead of %d Hz\n", sfset, wav.sf);
    }
    set_wav.sf = sfset;
  }
  return set_wav;
}

int getgcd(int a, int b)
{
  int m, n, l=1;
  
  if(a > b){
    m = a; n = b;
  }
  else{
    m = b; n = a;
  }

  while(l != 0){
    l = m % n;
    m = n;
    n = l;
  }
  return (a*b)/m;
}

void ptone(short sig[], int freq, int sf, int dur, float amp)
{
  int i;
  double amp_real, pi2f;
  
  pi2f = 2.0*PI*(double)freq/(double)sf;
  amp_real = pow(10.0, amp/20.0)*32767;
  for(i=0; i<dur; i++){
    sig[i] = sin(pi2f*(double)i)*amp_real;
  }
}

void mkptone(short sig[], int freq, int sf, int dur, float amp, float *phase)
{
  int i;
  double fpi, amp_real;

  fpi = 2.0*PI*(double)freq/(double)sf;
  amp_real = pow(10.0, amp/20.0)*32767;
  for(i=0; i<dur; i++){
    sig[i] = (short)(sin(fpi*(double)i+*phase)*amp_real);
  }
  *phase = fmod(fpi*dur+*phase, 2.0*PI);
}

/*
#define _USE_LRAND48
#define _USE_GAUSS
*/

void mkwnoise(short sig[], int dur, float amp)
{
  int i, j;
  double tmp, amp_real, div;

#ifdef _USE_GAUSS
  double gaussrand();
  amp_real = pow(10.0, (amp-3.0)/20.0)*32767;
#else
/*  18.6 meens to adjust wns level to pure tone. */
  amp_real = pow(10.0, (amp+18.6)/20.0)*32767;
  div = (double)RAND_MAX*RND_PRES;
#endif

  for(i=0; i<dur; i++){
    for(j=0, tmp=0.0; j<RND_PRES; j++){
#ifdef _USE_LRAND48
      tmp += lrand48();
#else
      tmp += rand();
#endif
#ifdef _USE_GAUSS
      sig[i] = gaussrand()*amp_real;
#endif
    }
#ifndef _USE_GAUSS
    sig[i] = (tmp/div - 0.5)*amp_real;
#endif
  }
}

void b16to8(short in[], unsigned char out[], int dur)
{
  long i;

  for(i=0; i<dur; i++){
    out[i] = (unsigned char)((in[i] >> 8) + 128);
  }
}

double gaussrand()
{
  static double V1, V2, S;
  static int phase = 0;
  double X;
  
  if(phase == 0) {
    do {
      double U1 = (double)rand() / RAND_MAX;
      double U2 = (double)rand() / RAND_MAX;
      
      V1 = 2 * U1 - 1;
      V2 = 2 * U2 - 1;
      S = V1 * V1 + V2 * V2;
    } while(S >= 1 || S == 0);
    
    X = V1 * sqrt(-2 * log(S) / S);
  } else
    X = V2 * sqrt(-2 * log(S) / S);
  
  phase = 1 - phase;
  
  return X;
}

int check_dsp(int dsp, int dsp_param, int value)
{
  int probe_value;

  probe_value = value;
  if(ioctl(dsp, dsp_param, &probe_value) == -1){
    fprintf(stderr, "Error. Can't change dsp parameter\n");
    exit(-1);
  }
  if(probe_value == value) return 1;
  else                     return 0;
}
    
